"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  WriteStreamError: () => WriteStreamError,
  chmod: () => chmod2,
  chown: () => chown2,
  constants: () => constants2,
  createReadStream: () => createReadStream2,
  createWriteStream: () => createWriteStream2,
  default: () => src_default,
  exists: () => exists,
  futimes: () => futimes,
  lstat: () => lstat,
  mkdir: () => mkdir,
  readFile: () => readFile,
  readdir: () => readdir,
  readlink: () => readlink,
  rename: () => rename,
  rmdir: () => rmdir,
  stat: () => stat,
  symlink: () => symlink,
  unlink: () => unlink,
  writeFile: () => writeFile
});
module.exports = __toCommonJS(src_exports);
var fs = __toESM(require("fs"), 1);
var import_ssh2_connect = require("ssh2-connect");
var constants2 = fs.constants;
async function chmod2(ssh, path, mode) {
  if (!ssh) {
    return fs.promises.chmod(path, mode);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        if (typeof path !== "string") path = path.toString();
        sftp.chmod(path, mode, (err2) => {
          sftp.end();
          if (err2) {
            reject(err2);
          } else {
            resolve();
          }
        });
      });
    });
  }
}
async function chown2(ssh, path, uid, gid) {
  if (!uid && !gid) {
    throw Error('Either option "uid" or "gid" is required');
  }
  if (!ssh) {
    return fs.promises.chown(path, uid, gid);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        if (typeof path !== "string") path = path.toString();
        sftp.chown(path, uid, gid, (err2) => {
          sftp.end();
          if (err2) {
            reject(err2);
          } else {
            resolve();
          }
        });
      });
    });
  }
}
async function createReadStream2(ssh, source, options = {}) {
  if (!ssh) {
    return Promise.resolve(fs.createReadStream(source, options));
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        if (typeof source !== "string") source = source.toString();
        const rs = sftp.createReadStream(
          source,
          options
        );
        rs.emit = /* @__PURE__ */ ((emit) => function(key, ...vals) {
          if (key === "error") {
            let val = vals[0];
            if (val.code === 4) {
              val = new Error(
                "EISDIR: illegal operation on a directory, read"
              );
              val.errno = -21;
              val.code = "EISDIR";
              val.syscall = "read";
            } else if (val.code === 2) {
              val = new Error(
                `ENOENT: no such file or directory, open '${source}'`
              );
              val.code = "ENOENT";
              val.errno = -2;
              val.syscall = "open";
              val.path = source.toString();
            } else if (val.code === 3) {
              val = new Error(`EACCES: permission denied, open '${source}'`);
              val.code = "EACCES";
              val.errno = -13;
              val.syscall = "open";
              val.path = source.toString();
            }
            return emit.call(rs, "error", val);
          }
          return emit.call(rs, key, ...vals);
        })(rs.emit);
        rs.on("close", () => sftp.end());
        resolve(rs);
      });
    });
  }
}
var WriteStreamError = class extends Error {
  code;
  errno;
  syscall;
  path;
};
async function createWriteStream2(ssh, path, options = {}) {
  if (typeof options === "string") options = { encoding: options };
  if (!ssh) {
    return Promise.resolve(fs.createWriteStream(path, options));
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        if (typeof path !== "string") path = path.toString();
        const ws = sftp.createWriteStream(
          path,
          options
        );
        ws.emit = /* @__PURE__ */ ((emit) => function(key, ...vals) {
          let val = vals[0];
          if (key === "error") {
            if (val.code === 4) {
              val = new WriteStreamError(
                `EISDIR: illegal operation on a directory, open '${path}'`
              );
              val.errno = -21;
              val.code = "EISDIR";
              val.syscall = "open";
            } else if (val.code === 2) {
              val = new WriteStreamError(
                `ENOENT: no such file or directory, open '${path}'`
              );
              val.code = "ENOENT";
              val.errno = -2;
              val.syscall = "open";
              val.path = path.toString();
            }
            return emit.call(ws, "error", val);
          }
          return emit.call(ws, key, val);
        })(ws.emit);
        ws.on("close", () => sftp.end());
        resolve(ws);
      });
    });
  }
}
async function exists(ssh, path) {
  if (!ssh) {
    try {
      await fs.promises.access(path, fs.constants.F_OK);
      return true;
    } catch {
      return false;
    }
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        if (typeof path !== "string") path = path.toString();
        sftp.stat(path, (err2) => {
          sftp.end();
          if (err2 && err2.code !== 2 && err2.type !== "NO_SUCH_FILE")
            return reject(err2);
          resolve(err2 ? false : true);
        });
      });
    });
  }
}
async function futimes(ssh, path, atime, mtime) {
  if (!ssh) {
    return fs.promises.utimes(path, atime, mtime);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        const end = (err2) => {
          sftp.end();
          if (err2) {
            reject(err2);
          } else {
            resolve();
          }
        };
        if (typeof path !== "string") path = path.toString();
        sftp.open(path, "r", (err2, fd) => {
          if (err2) return end(err2);
          if (typeof atime === "string") atime = parseInt(atime, 10);
          if (typeof mtime === "string") mtime = parseInt(mtime, 10);
          sftp.futimes(fd, atime, mtime, (err3) => {
            if (err3) return end(err3);
            sftp.close(fd, (err4) => {
              end(err4);
            });
          });
        });
      });
    });
  }
}
async function lstat(ssh, path) {
  if (!ssh) {
    return fs.promises.lstat(path);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        if (typeof path !== "string") path = path.toString();
        sftp.lstat(
          path,
          (err2, attr) => {
            sftp.end();
            if (err2 && (err2.type == "NO_SUCH_FILE" || err2.code === 2)) {
              err2.code = "ENOENT";
              return reject(err2);
            }
            resolve(attr);
          }
        );
      });
    });
  }
}
async function mkdir(ssh, path, options) {
  if (typeof options !== "object") options = { mode: options };
  if (typeof options.mode === "string")
    options.mode = parseInt(options.mode, 8);
  if (!ssh) {
    return fs.promises.mkdir(path, options.mode).then(() => {
      if (options.uid && options.gid)
        return fs.promises.chown(path, options.uid, options.gid);
    });
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        if (typeof path !== "string") path = path.toString();
        Promise.resolve(path).then(
          (path2) => new Promise(
            (resolve2, reject2) => sftp.mkdir(
              path2,
              options,
              (err2) => {
                if (err2?.message === "Failure") {
                  err2 = new Error(
                    `EEXIST: file already exists, mkdir '${path2}'`
                  );
                  err2.errno = -17;
                  err2.code = "EEXIST";
                  err2.path = path2.toString();
                  err2.syscall = "mkdir";
                }
                if (err2) return reject2(err2);
                resolve2(path2);
              }
            )
          )
        ).then(
          (path2) => new Promise((resolve2, reject2) => {
            if (!options.uid || !options.gid) return resolve2();
            sftp.chown(path2, options.uid, options.gid, (err2) => {
              if (err2) {
                reject2(err2);
              } else {
                resolve2();
              }
            });
          })
        ).then(resolve).catch(reject).finally(() => sftp.end());
      });
    });
  }
}
async function readdir(ssh, path) {
  if (!ssh) {
    return fs.promises.readdir(path);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        const not_a_dir = (path2, err2) => {
          sftp.stat(path2, (er, attr) => {
            if (!er && !attr.isDirectory()) {
              err2 = Error(`ENOTDIR: not a directory, scandir '${path2}'`);
              err2.errno = 27;
              err2.code = "ENOTDIR";
              err2.path = path2;
            }
            reject(err2);
          });
        };
        if (typeof path !== "string") path = path.toString();
        sftp.opendir(path, (err2, handle) => {
          if (typeof path !== "string") path = path.toString();
          if (err2) return not_a_dir(path, err2);
          sftp.readdir(handle, (err1, files) => {
            sftp.close(handle, (err22) => {
              if (err1 || err22) return reject(err1 || err22);
              sftp.end();
              resolve(files.map((file) => file.filename));
            });
          });
        });
      });
    });
  }
}
async function readFile(ssh, path, options = {}) {
  if (typeof options === "string") options = { encoding: options };
  if (!path) throw Error(`Invalid path '${path}'`);
  if (!ssh) {
    return fs.promises.readFile(path, options);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        const buffers = [];
        if (typeof path !== "string") path = path.toString();
        const s = sftp.createReadStream(path, {
          ...options,
          encoding: void 0
        });
        const finish = (err2, data) => {
          sftp.end();
          if (err2 || data == null) {
            reject(err2);
          } else {
            resolve(
              options?.encoding ? data.toString(options?.encoding) : data
            );
          }
        };
        s.on("data", (buffer) => {
          buffers.push(buffer);
        });
        s.on("error", (err2) => {
          if (err2.code === 4) {
            err2 = new Error("EISDIR: illegal operation on a directory, read");
            err2.errno = -21;
            err2.code = "EISDIR";
            err2.syscall = "read";
          } else if (err2.code === 2) {
            err2 = new Error(
              `ENOENT: no such file or directory, open '${path}'`
            );
            err2.code = "ENOENT";
            err2.errno = -2;
            err2.syscall = "open";
            err2.path = path.toString();
          }
          finish(err2);
        });
        s.on(
          "end",
          () => finish(
            null,
            Buffer.concat(
              buffers.map(
                (buf) => new Uint8Array(buf.buffer, buf.byteOffset, buf.length)
              )
            )
          )
        );
      });
    });
  }
}
async function readlink(ssh, path) {
  if (!ssh) {
    return fs.promises.readlink(path);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        if (typeof path !== "string") path = path.toString();
        sftp.readlink(path, (err2, target) => {
          sftp.end();
          if (err2) {
            reject(err2);
          } else {
            resolve(target);
          }
        });
      });
    });
  }
}
async function rename(ssh, source, target) {
  if (typeof target !== "string") target = target.toString();
  if (typeof source !== "string") source = source.toString();
  if (!ssh) {
    return fs.promises.rename(source, target);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        sftp.unlink(target, () => {
          sftp.rename(source, target, (err2) => {
            sftp.end();
            if (err2) {
              reject(err2);
            } else {
              resolve();
            }
          });
        });
      });
    });
  }
}
async function rmdir(ssh, target) {
  if (typeof target !== "string") target = target.toString();
  if (!ssh) {
    return fs.promises.rmdir(target);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        sftp.rmdir(target, (err2) => {
          sftp.end();
          if (err2 && (err2.type === "NO_SUCH_FILE" || err2.code === 2)) {
            err2.message = `ENOENT: no such file or directory, rmdir '${target}'`;
            err2.errno = -2;
            err2.code = "ENOENT";
            err2.syscall = "rmdir";
            err2.path = target;
          }
          if (err2) {
            reject(err2);
          } else {
            resolve();
          }
        });
      });
    });
  }
}
async function stat(ssh, path) {
  if (typeof path !== "string") path = path.toString();
  if (!ssh) {
    return fs.promises.stat(path);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        sftp.stat(path, (err2, attr) => {
          sftp.end();
          if (err2 && (err2.type === "NO_SUCH_FILE" || err2.code === 2)) {
            err2.code = "ENOENT";
            return reject(err2);
          }
          if (err2) {
            reject(err2);
          } else {
            resolve(attr);
          }
        });
      });
    });
  }
}
async function symlink(ssh, srcpath, dstpath) {
  if (typeof srcpath !== "string") srcpath = srcpath.toString();
  if (typeof dstpath !== "string") dstpath = dstpath.toString();
  if (!ssh) {
    return fs.promises.symlink(srcpath, dstpath);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        sftp.symlink(srcpath, dstpath, (err2) => {
          sftp.end();
          if (err2) {
            reject(err2);
          } else {
            resolve();
          }
        });
      });
    });
  }
}
async function unlink(ssh, path) {
  if (typeof path !== "string") path = path.toString();
  if (!ssh) {
    return fs.promises.unlink(path);
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      ssh.sftp((err, sftp) => {
        if (err) return reject(err);
        sftp.unlink(path, (err2) => {
          sftp.end();
          if (err2) {
            reject(err2);
          } else {
            resolve();
          }
        });
      });
    });
  }
}
async function writeFile(ssh, target, data, options = {}) {
  if (typeof target !== "string") target = target.toString();
  if (typeof options === "string") options = { encoding: options };
  if (!ssh) {
    return new Promise((resolve, reject) => {
      let error = false;
      const write = () => {
        if (typeof data !== "string" && !Buffer.isBuffer(data))
          data.on("error", (err) => {
            reject(err);
            error = true;
          });
        const stream = fs.createWriteStream(target, options);
        if (typeof data === "string" || Buffer.isBuffer(data)) {
          if (data) stream.write(data);
          stream.end();
        } else {
          data.pipe(stream);
        }
        stream.on("error", (err) => {
          if (!error) reject(err);
        });
        stream.on("end", () => stream.destroy());
        stream.on("close", () => chown3());
      };
      const chown3 = () => {
        if (!options.uid || !options.gid) {
          return chmod3();
        }
        fs.chown(target, options.uid, options.gid, (err) => {
          if (err) return reject(err);
          chmod3();
        });
      };
      const chmod3 = () => {
        if (!options.mode) return finish();
        fs.chmod(target, options.mode, (err) => {
          finish(err);
        });
      };
      const finish = (err) => {
        if (!error) {
          if (err) {
            reject(err);
          } else {
            resolve();
          }
        }
      };
      write();
    });
  } else {
    return new Promise((resolve, reject) => {
      if (!(0, import_ssh2_connect.opened)(ssh)) return reject(Error("Closed SSH Connection"));
      let error = false;
      if (typeof data !== "string" && !Buffer.isBuffer(data))
        data.on("error", (err) => {
          reject(err);
          error = true;
        });
      ssh.sftp((err, sftp) => {
        if (err) {
          if (!error) reject(err);
          error = true;
          return;
        }
        const write = () => {
          const ws = sftp.createWriteStream(
            target,
            options
          );
          if (typeof data === "string" || Buffer.isBuffer(data)) {
            if (data) ws.write(data);
            ws.end();
          } else {
            data.pipe(ws);
          }
          ws.on("error", (err2) => finish(err2));
          ws.on("end", () => ws.destroy());
          ws.on("close", () => chown3());
        };
        const chown3 = () => {
          if (!options.uid || !options.gid) return chmod3();
          sftp.chown(target, options.uid, options.gid, (err2) => {
            if (err2) return finish(err2);
            chmod3();
          });
        };
        const chmod3 = () => {
          if (!options.mode) return finish();
          sftp.chmod(target, options.mode, (err2) => {
            finish(err2);
          });
        };
        const finish = (err2) => {
          sftp.end();
          if (!error) {
            if (err2) {
              reject(err2);
            } else {
              resolve();
            }
          }
          error = true;
        };
        write();
      });
    });
  }
}
var src_default = {
  constants: constants2,
  chmod: chmod2,
  chown: chown2,
  createReadStream: createReadStream2,
  WriteStreamError,
  createWriteStream: createWriteStream2,
  exists,
  futimes,
  lstat,
  mkdir,
  readdir,
  readFile,
  readlink,
  rename,
  rmdir,
  stat,
  symlink,
  unlink,
  writeFile
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  WriteStreamError,
  chmod,
  chown,
  constants,
  createReadStream,
  createWriteStream,
  exists,
  futimes,
  lstat,
  mkdir,
  readFile,
  readdir,
  readlink,
  rename,
  rmdir,
  stat,
  symlink,
  unlink,
  writeFile
});
//# sourceMappingURL=index.cjs.map